import java.awt.Color;

import javax.swing.JFrame;
import javax.swing.SwingUtilities;

import com.mindfusion.common.DateTime;
import com.mindfusion.drawing.Colors;
import com.mindfusion.drawing.DashStyle;
import com.mindfusion.drawing.SolidBrush;
import com.mindfusion.spreadsheet.Cell;
import com.mindfusion.spreadsheet.CellChangedEvent;
import com.mindfusion.spreadsheet.CellValidationEvent;
import com.mindfusion.spreadsheet.ComparisonOperator;
import com.mindfusion.spreadsheet.ConditionalFormatType;
import com.mindfusion.spreadsheet.HorizontalAlignment;
import com.mindfusion.spreadsheet.IConditionalFormat;
import com.mindfusion.spreadsheet.IStyle;
import com.mindfusion.spreadsheet.IValidation;
import com.mindfusion.spreadsheet.InplaceEditValidationEvent;
import com.mindfusion.spreadsheet.Measure;
import com.mindfusion.spreadsheet.ValidationErrorAction;
import com.mindfusion.spreadsheet.ValidationType;
import com.mindfusion.spreadsheet.VerticalAlignment;
import com.mindfusion.spreadsheet.Workbook;
import com.mindfusion.spreadsheet.WorkbookAdapter;
import com.mindfusion.spreadsheet.Worksheet;
import com.mindfusion.spreadsheet.WorksheetView;
import com.mindfusion.spreadsheet.WorksheetViewAdapter;


public class ProjectPlanner
{
	public static void main(String[] args)
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				MainWindow window = new MainWindow();
				window.setTitle("JSpreadsheet Sample: Project Planner");
				window.setVisible(true);
			}
		});
	}
	
	public static class MainWindow extends JFrame
	{
		public MainWindow()
		{
			setDefaultCloseOperation(EXIT_ON_CLOSE);
			setSize(1280, 1024);

			workbook = new Workbook();

			view = new WorksheetView();
			add(view);

			view.setAllowMoveCells(false);
			view.setAllowMoveHeaders(false);

			view.addViewListener(new WorksheetViewAdapter()
			{
				@Override
				public void inplaceEditStarting(InplaceEditValidationEvent e)
				{
					if (ignoreChanges)
						return;
					if (!(e.getItem() instanceof Cell))
						return;
					Cell cell = (Cell)e.getItem();
					if (cell == null)
						return;
					
					if (cell.getColumn() == 1 && (cell.getRow() == 0 || cell.getRow() == 1))
						return;

					if (cell.getColumn() <= 2 && cell.getRow() >= 5 && cell.getRow() <= 5 + projectPhases)
						return;

					e.setCancel(true);
				}
			});
			workbook.addWorkbookListener(new WorkbookAdapter()
			{
				@Override
				public void worksheetCellChanged(CellChangedEvent e)
				{
					if (ignoreChanges)
						return;
					ignoreChanges = true;
					Worksheet sheet = workbook.getWorksheets().get(0);
					if (projectPhases + 5 == e.getCell().getRow() && entryIsValid(e.getCell().getRow()))
					{
						sheet.getRows().get(projectPhases + 6).setIsHidden(false);
						projectPhases++;
						if (projectPhases > 1)
						{
							sheet.beginInit();
							sheet.getRows().insert(e.getCell().getRow() + 1);
							sheet.getCellRanges().get(0, e.getCell().getRow() + 1, 2, e.getCell().getRow() + 1).getStyle().setBackground(new SolidBrush(Colors.AntiqueWhite));
							for (int c = 3; c <= 15; c++)
							{
								Cell headerCell = sheet.getCells().get(c, 4);
								Cell cell = sheet.getCells().get(c, e.getCell().getRow() + 1);
								cell.setData(" ");
								cell.getStyle().getConditionalFormats().clear();
								IConditionalFormat cf = cell.getStyle().getConditionalFormats().add();
								cf.setType(ConditionalFormatType.Expression);
								cf.setFirst(String.format("=AND(NOT(ISBLANK(B%2$d)),NOT(ISBLANK(C%2$d)),MONTH(B%2$d)<=%1$d,MONTH(C%2$d)>=%1$d,YEAR(B%2$d)=%3$d,YEAR(C%2$d)=%3$d)",
									((DateTime)headerCell.getData()).getMonth(), e.getCell().getRow() + 2, ((DateTime)headerCell.getData()).getYear()));
								cf.getStyle().setBackground(new SolidBrush(Colors.SkyBlue));
							}
							sheet.endInit();
							
						}
						view.getSelection().set(0, e.getCell().getRow());
					}

					int rangeCount = 0;
					for (@SuppressWarnings("unused") Cell range: sheet.getCellRanges().get(1, 5, 2, 0x100000))
						rangeCount++;
					if (rangeCount >= 2)
					{
						int maxRow = 0;
						for (Cell cell: sheet.getCellRanges().get(0, 0, 2, 0x100000))
							maxRow = Math.max(maxRow, cell.getRow());
						sheet.getCells().get(1, maxRow).setData(String.format("%1$d days", getProjectLength()));
					}
					ignoreChanges = false;
				}

				@Override
				public void worksheetCellChanging(CellValidationEvent e)
				{
					if (ignoreChanges)
						return;

					// If the currency changed 
					if (e.getCell().getColumn() == 1 &&
						(e.getCell().getRow() == 0 || e.getCell().getRow() == 1 || e.getCell().getRow() == 2))
						return;

					if (e.getNewData() != null &&
						e.getCell().getColumn() <= 2 &&
						e.getCell().getRow() >= 5 &&
						e.getCell().getRow() <= 5 + projectPhases)
						return;

					e.setCancel(true);
				}
			});

			createProjectTemplate();
			projectStart = new DateTime(DateTime.today().getYear(), 1, 1);
			projectEnd = new DateTime(DateTime.today().getYear(), 12, 31);
			workbook.getWorksheets().get(0).getCells().get("B3").setData(projectStart.toLongDateString());

			applyDateValidation(workbook.getWorksheets().get(0).getCellRanges().get(1, 5, 2, 0xfffff).getStyle());
		}

		private void createProjectTemplate()
		{
			workbook.getWorksheets().clear();
			Worksheet sheet = workbook.getWorksheets().add("Project");
			view.setWorksheet(sheet);

			ignoreChanges = true;
			projectPhases = 0;

			sheet.beginInit();

			sheet.getColumns().get(0).setWidth(Measure.point(120));
			sheet.getColumns().get(1, 2).setWidth(Measure.point(50));
			sheet.getColumns().get(3, 14).setWidth(Measure.point(50));

			sheet.getRows().get(0).setHeight(Measure.point(19));

			sheet.getCellRanges().get(1, 0, 14, 0).merge();
			sheet.getCellRanges().get(1, 1, 14, 1).merge();
			sheet.getCellRanges().get(1, 2, 14, 2).merge();
			sheet.getCellRanges().get(1, 3, 2, 3).merge();
			sheet.getCellRanges().get(3, 3, 5, 3).merge();
			sheet.getCellRanges().get(6, 3, 8, 3).merge();
			sheet.getCellRanges().get(9, 3, 11, 3).merge();
			sheet.getCellRanges().get(12, 3, 14, 3).merge();
			sheet.getCellRanges().get(1, 7, 14, 7).merge();

			sheet.getColumns().get(15, 0x3ff).setIsHidden(true);
			sheet.getRows().get(8, 0xfffff).setIsHidden(true);
			sheet.getRows().get(6).setIsHidden(true);

			IStyle style = sheet.getRows().get(0, 2).getStyle();
			style.setVerticalAlignment(VerticalAlignment.Middle);

			style = sheet.getRows().get(3, 4).getStyle();
			style.setFontBold(true);
			style.setHorizontalAlignment(HorizontalAlignment.Center);

			style = sheet.getCellRanges().get(1, 0, 1, 1).getStyle();
			style.setBackground(new SolidBrush(new Color(102, 154, 204)));

			sheet.getRows().get(4).getStyle().setBackground(new SolidBrush(Colors.LightGray));
			sheet.getRows().get(7).getStyle().setBackground(new SolidBrush(Colors.LightGray));
			sheet.getRows().get(7).getStyle().setFontBold(true);

			sheet.getCells().get("B1").getStyle().setFontBold(true);
			sheet.getCells().get("B1").getStyle().setFontSize(12.0);
			sheet.getCells().get("B2").getStyle().setFontItalic(true);
			sheet.getCellRanges().get(0, 0, 0, 3).getStyle().setHorizontalAlignment(HorizontalAlignment.Right);
			sheet.getCellRanges().get(0, 0, 0, 3).getStyle().setPaddingRight(Measure.point(5));

			sheet.getCellRanges().get(1, 0, 1, 2).getStyle().setHorizontalAlignment(HorizontalAlignment.Center);
			sheet.getCells().get("A8").getStyle().setHorizontalAlignment(HorizontalAlignment.Right);
			sheet.getCells().get("A8").getStyle().setPaddingRight(Measure.point(5));
			sheet.getCells().get("B8").getStyle().setPaddingLeft(Measure.point(5));

			applySimpleBorders(sheet.getCellRanges().get(0, 4, 14, 7).getStyle());

			sheet.getCells().get("A1").setData("Project name");
			sheet.getCells().get("A2").setData("Project author");
			sheet.getCells().get("A3").setData("Project start");

			sheet.getCells().get("D4").setData("I Quarter");
			sheet.getCells().get("G4").setData("II Quarter");
			sheet.getCells().get("J4").setData("III Quarter");
			sheet.getCells().get("M4").setData("IV Quarter");

			sheet.getCells().get("A5").setData("Project phase");
			sheet.getCells().get("B5").setData("Start date");
			sheet.getCells().get("C5").setData("End date");

			sheet.getCellRanges().get(0, 5, 2, 6).getStyle().setBackground(new SolidBrush(new Color (102, 154, 204)));

			sheet.getCells().get("A8").setData("Project duration");

			DateTime d = new DateTime(DateTime.today().getYear(), 1, 1);
			for (int c = 3; c <= 15; c++)
			{
				Cell cell = sheet.getCells().get(c, 4);
				cell.setData(d);
				cell.getStyle().setFormat("MMM yyyy");
				d = d.addMonths(1);
			}

			for (int c = 3; c <= 15; c++)
			{
				Cell headerCell = sheet.getCells().get(c, 4);
				Cell cell = sheet.getCells().get(c, 5);
				cell.setData(" ");
				IConditionalFormat cf = cell.getStyle().getConditionalFormats().add();
				cf.setType(ConditionalFormatType.Expression);
				cf.setFirst(String.format("=AND(NOT(ISBLANK(B6)),NOT(ISBLANK(C6)),MONTH(B6)<=%1$d,MONTH(C6)>=%1$d)", ((DateTime)headerCell.getData()).getMonth()));
				cf.getStyle().setBackground(new SolidBrush(Colors.SkyBlue));
			}
			for (int c = 3; c <= 15; c++)
			{
				Cell headerCell = sheet.getCells().get(c, 4);
				Cell cell = sheet.getCells().get(c, 6);
				cell.setData(" ");
				IConditionalFormat cf = cell.getStyle().getConditionalFormats().add();
				cf.setType(ConditionalFormatType.Expression);
				cf.setFirst(String.format("=AND(NOT(ISBLANK(B7)),NOT(ISBLANK(C7)),MONTH(B7)<=%1$d,MONTH(C7)>=%1$d)", ((DateTime)headerCell.getData()).getMonth()));
				cf.getStyle().setBackground(new SolidBrush(Colors.SkyBlue));
			}

			sheet.endInit();

			ignoreChanges = false;
		}

		private int getProjectLength()
		{
			DateTime min = DateTime.MaxValue;
			DateTime max = DateTime.MinValue;
			for (Cell cell: workbook.getWorksheets().get(0).getCellRanges().get(1, 5, 1, 0x100000))
			{
				if (cell.getData() == null || !(cell.getValue() instanceof DateTime))
					continue;
				DateTime date = (DateTime)cell.getValue();
				if (DateTime.op_LessThanOrEqual(date, min))
					min = date;
			}
			for (Cell cell: workbook.getWorksheets().get(0).getCellRanges().get(2, 5, 2, 0x100000))
			{
				if (cell.getData() == null || !(cell.getValue() instanceof DateTime))
					continue;
				DateTime date = (DateTime)cell.getValue();
				if (DateTime.op_GreaterThanOrEqual(date, max))
					max = date;
			}
			return (int)(max.subtract(min)).getTotalDays();
		}

		private boolean entryIsValid(int rowIndex)
		{
			Worksheet sheet = workbook.getWorksheets().get(0);
			if (sheet.getCells().get(0, rowIndex).getData() != null &&
				sheet.getCells().get(1, rowIndex).getData() != null &&
				sheet.getCells().get(2, rowIndex).getData() != null)
				return true;
			return false;
		}

		private void applySimpleBorders(IStyle style)
		{
			style.setBorderBottomColor(Colors.Black);
			style.setBorderLeftColor(Colors.Black);
			style.setBorderRightColor(Colors.Black);
			style.setBorderTopColor(Colors.Black);
			style.setBorderBottomSize(Measure.point(1));
			style.setBorderLeftSize(Measure.point(1));
			style.setBorderRightSize(Measure.point(1));
			style.setBorderTopSize(Measure.point(1));
			style.setBorderBottomStyle(DashStyle.Solid);
			style.setBorderLeftStyle(DashStyle.Solid);
			style.setBorderRightStyle(DashStyle.Solid);
			style.setBorderTopStyle(DashStyle.Solid);
		}

		private void applyDateValidation(IStyle style)
		{
			IValidation val = style.getValidation();
			val.setType(ValidationType.Date);
			val.setAllowBlankCells(false);
			val.setErrorAction(ValidationErrorAction.Stop);
			val.setErrorMessage(String.format("Enter a valid date in the range %1$s - %2$s in the cell.", projectStart.toShortDateString(), projectEnd.toShortDateString()));
			val.setErrorTitle("Error");
			val.setFirst(projectStart.toShortDateString());
			val.setSecond(projectEnd.toShortDateString());
			val.setOperator(ComparisonOperator.Between);
			val.setShowError(true);
		}


		private static final long serialVersionUID = 1L;

		private int projectPhases;
		private DateTime projectStart;
		private DateTime projectEnd;
		private boolean ignoreChanges;

		private WorksheetView view;
		private Workbook workbook;
	}
}